#  SPDX-License-Identifier: Apache-2.0
"""
Constants used by subarulink to interact with the STARLINK HTTP API.

This is an undocumented API derived from analysis of the MySubaru Android app v2.7.30.
"""
